@@ -22,7 +22,9 @@ import logging
 import os
 import sys
 
+from novaclient import api_versions
 from novaclient.v2 import servers
+from openstack import exceptions as sdk_exceptions
 from osc_lib.cli import parseractions
 from osc_lib.command import command
 from osc_lib import exceptions
@@ -247,13 +249,16 @@ class AddFloatingIP(network_common.NetworkAndComputeCommand):
         parser.add_argument(
             "ip_address",
             metavar="<ip-address>",
-            help=_("Floating IP address to assign to server (IP only)"),
+            help=_("Floating IP address to assign to the first available "
+                   "server port (IP only)"),
         )
         parser.add_argument(
             "--fixed-ip-address",
             metavar="<ip-address>",
             help=_(
-                "Fixed IP address to associate with this floating IP address"
+                "Fixed IP address to associate with this floating IP address. "
+                "The first server port containing the fixed IP address will "
+                "be used"
             ),
         )
         return parser
@@ -270,12 +275,45 @@ class AddFloatingIP(network_common.NetworkAndComputeCommand):
             compute_client.servers,
             parsed_args.server,
         )
-        port = list(client.ports(device_id=server.id))[0]
-        attrs['port_id'] = port.id
+        ports = list(client.ports(device_id=server.id))
+        # If the fixed IP address was specified, we need to find the
+        # corresponding port.
         if parsed_args.fixed_ip_address:
-            attrs['fixed_ip_address'] = parsed_args.fixed_ip_address
-
-        client.update_ip(obj, **attrs)
+            fip_address = parsed_args.fixed_ip_address
+            attrs['fixed_ip_address'] = fip_address
+            for port in ports:
+                for ip in port.fixed_ips:
+                    if ip['ip_address'] == fip_address:
+                        attrs['port_id'] = port.id
+                        break
+                else:
+                    continue
+                break
+            if 'port_id' not in attrs:
+                msg = _('No port found for fixed IP address %s')
+                raise exceptions.CommandError(msg % fip_address)
+            client.update_ip(obj, **attrs)
+        else:
+            # It's possible that one or more ports are not connected to a
+            # router and thus could fail association with a floating IP.
+            # Try each port until one succeeds. If none succeed, re-raise the
+            # last exception.
+            error = None
+            for port in ports:
+                attrs['port_id'] = port.id
+                try:
+                    client.update_ip(obj, **attrs)
+                except sdk_exceptions.NotFoundException as exp:
+                    # 404 ExternalGatewayForFloatingIPNotFound from neutron
+                    LOG.info('Skipped port %s because it is not attached to '
+                             'an external gateway', port.id)
+                    error = exp
+                    continue
+                else:
+                    error = None
+                    break
+            if error:
+                raise error
 
     def take_action_compute(self, client, parsed_args):
         client.api.floating_ip_add(
@@ -754,9 +792,14 @@ class CreateServer(command.ShowOne):
                     raise exceptions.CommandError(msg)
                 nics = nics[0]
         else:
-            # Default to empty list if nothing was specified, let nova side to
-            # decide the default behavior.
-            nics = []
+            # Compute API version >= 2.37 requires a value, so default to
+            # 'auto' to maintain legacy behavior if a nic wasn't specified.
+            if compute_client.api_version >= api_versions.APIVersion('2.37'):
+                nics = 'auto'
+            else:
+                # Default to empty list if nothing was specified, let nova
+                # side to decide the default behavior.
+                nics = []
 
         # Check security group exist and convert ID to name
         security_group_names = []
@@ -1315,11 +1358,13 @@ class MigrateServer(command.Command):
             parsed_args.server,
         )
         if parsed_args.live:
-            server.live_migrate(
-                host=parsed_args.live,
-                block_migration=parsed_args.block_migration,
-                disk_over_commit=parsed_args.disk_overcommit,
-            )
+            kwargs = {
+                'host': parsed_args.live,
+                'block_migration': parsed_args.block_migration
+            }
+            if compute_client.api_version < api_versions.APIVersion('2.25'):
+                kwargs['disk_over_commit'] = parsed_args.disk_overcommit
+            server.live_migrate(**kwargs)
         else:
             if parsed_args.block_migration or parsed_args.disk_overcommit:
                 raise exceptions.CommandError("--live must be specified if "
@@ -1331,6 +1376,7 @@ class MigrateServer(command.Command):
             if utils.wait_for_status(
                 compute_client.servers.get,
                 server.id,
+                success_status=['active', 'verify_resize'],
                 callback=_show_progress,
             ):
                 sys.stdout.write(_('Complete\n'))
